import { Injectable } from '@angular/core';
import { TreeviewSelection } from './treeview-item';

@Injectable()
export abstract class TreeviewI18n {
  abstract getText(selection: TreeviewSelection): string; // 获取文本
  abstract getAllCheckboxText(): string; // 获取全选文本
  abstract getFilterPlaceholder(): string; // 获取筛选提示
  abstract getFilterNoItemsFoundText(): string; // 暂无内容
  abstract getTooltipCollapseExpandText(isCollapse: boolean): string; // 展开
}

@Injectable()
export class DefaultTreeviewI18n extends TreeviewI18n {
  getText(selection: TreeviewSelection): string {
    if (selection.uncheckedItems.length === 0) {
      if (selection.checkedItems.length > 0) {
        return this.getAllCheckboxText();
      } else {
        return '';
      }
    }

    switch (selection.checkedItems.length) {
      case 0:
        return '请选择';
      case 1:
        return selection.checkedItems[0].text;
      case 2:
        return (
          selection.checkedItems[0].text +
          '、' +
          selection.checkedItems[1].text
        );
      default:
        return `${selection.checkedItems.length} options selected`;
    }
  }

  getAllCheckboxText(): string {
    return 'All';
  }

  getFilterPlaceholder(): string {
    return '请输入';
  }

  getFilterNoItemsFoundText(): string {
    return '暂无数据';
  }

  getTooltipCollapseExpandText(isCollapse: boolean): string {
    return isCollapse ? 'Expand' : 'Collapse';
  }
}
